from py3db.log.Log import Log
from pymysql import connect


class MySql:
    def __init__(self, ip, user_name, password, database, table=None):
        self.ip = ip
        self.user_name = user_name
        self.password = password
        self.database = database
        self.db = None
        self.table_name = None
        self.log = Log("mysql.log")
        self.create_connect()
        self.results = []
        self.op_list = []

    # @classmethod
    # def table(cls, table_name):
    #     cls.table_name = table_name

    def query(self, sql):
        cursor = self.db.cursor()
        cursor.execute(sql)
        return cursor.fetchall()

    def __str__(self) -> str:
        return "\n".join(["%s:%s" % item for item in self.__dict__.items()])

    def create_connect(self):
        try:
            self.db = connect(
                host=self.ip,
                user=self.user_name,
                password=self.password,
                db=self.database,
                charset='utf8mb4'
            )
            if not self.db:
                self.log.call_error()
        except Exception as e:
            self.log.call_error()
            self.log.error(str(e))

    def close_connect(self):
        try:
            self.db.close()
        except Exception as e:
            self.log.call_error()
            self.log.error(str(e))

    def operation_database(self, sql, variable_name=None, variable=None):
        cursor = self.db.cursor()
        try:
            cursor.execute(sql)
            self.db.commit()
        except Exception as e:
            self.log.call_error()
            self.log.error("{0} {1} {2}".format(
                e, variable_name, variable
            ))

    def insert_one(self, table=None, columns_list=[]):
        if not columns_list:
            print('无对应数据')
        else:
            self.table_name = table if table else self.table_name
            value_str = "({})".format(
                ",".join(list(map(lambda x: f"'{x}'" if x else "NULL", columns_list))))
            self.op_list = ['insert', 'into',
                            self.table_name, 'values', value_str]
        sql = " ".join(self.op_list)
        self.operation_database(sql)

    def update(self, table=None, update_dict: dict = {}, condition_dict: dict = {}, equal_str="="):
        update_list = []
        condition_list = []
        self.table_name = table if table else self.table_name
        for key, item in update_dict.items():
            item = item if item else "NULL"
            update_list.append('%s=%s' % (key, item))
        update_sql = ",".join(update_list)

        for key, item in condition_dict.items():
            item = item if item else "NULL"
            condition_list.append("`%s` %s '%s'" % (key, equal_str, item))
        condition_sql = "1=1" if condition_dict == {
        } else ",".join(condition_list)

        sql = "update {} set {} where {}".format(
            self.table_name, update_sql, condition_sql)
        self.operation_database(sql)

    def delete(self, table=None, condition_dict={}, equal_str="="):
        condition_list = []
        self.table_name = table if table else self.table_name
        for key, item in condition_dict.items():
            item = item if item else "NULL"
            condition_list.append("`%s` %s '%s'" % (key, equal_str, item))
        condition_sql = ",".join(condition_list)
        delete_sql = "delete from %s where %s" % (
            self.table_name, condition_sql)
        self.operation_database(delete_sql)

    def __del__(self):
        if not self.db and not self:
            self.db.close_connect()


# if __name__ == "__main__":
#     mysql = MySql("localhost", "root", "123456", "sex")
#     mysql.delete("scontent", {"a": 10})
